@@ -2,7 +2,7 @@
 ###########################################################
 ## EMode - Python interface, by EMode Photonix LLC
 ###########################################################
-## Copyright (c) 2022 EMode Photonix LLC
+## Copyright (c) 2023 EMode Photonix LLC
 ###########################################################
 
 import os, socket, json, pickle, time, atexit, base64, struct
@@ -12,22 +12,27 @@ import numpy as np
 import scipy.io as sio
 
 class EMode:
-    def __init__(self, sim="emode", verbose=False, roaming=False, open_existing=False, new_name=False, priority='pN'):
+    def __init__(self, sim="emode", simulation_name=None, save_path='.', verbose=False, roaming=False, open_existing=False, new_name=False, priority='pN'):
         '''
         Initialize defaults and connects to EMode.
         '''
         self.status = 'open'
         atexit.register(self.close)
-        try:
-            sim = str(sim)
-        except:
+        
+        if (simulation_name != None): sim = simulation_name
+        
+        if not type(sim) == str:
             raise TypeError("input parameter 'sim' must be a string")
             return
-        try:
-            priority = str(priority)
-        except:
+        
+        if not type(save_path) == str:
+            raise TypeError("input parameter 'save_path' must be a string")
+            return
+        
+        if not type(priority) == str:
             raise TypeError("input parameter 'priority' must be a string")
             return
+        
         self.dsim = sim
         self.ext = ".eph"
         self.exit_flag = False
@@ -80,11 +85,13 @@ class EMode:
         time.sleep(0.1) # wait for EMode
         
         if (open_existing):
-            RV = self.call("EM_open", sim=sim, new_name=new_name)
+            RV = self.call("EM_open", sim=sim, save_path=save_path, new_name=new_name)
         else:
-            RV = self.call("EM_init", sim=sim)
+            RV = self.call("EM_init", sim=sim, save_path=save_path)
+        
         if (RV == 'ERROR'):
             raise RuntimeError("internal EMode error")
+        
         self.dsim = RV[len("sim:"):]
         return
     
@@ -109,8 +116,8 @@ class EMode:
             
             sendset[kw] = data
         
-        if ('sim' not in kwargs):
-            sendset['sim'] = self.dsim
+        if ('sim' not in kwargs) and ('simulation_name' not in kwargs):
+            sendset['simulation_name'] = self.dsim
         
         try:
             sendstr = json.dumps(sendset)
@@ -137,26 +144,6 @@ class EMode:
         
         return result
     
-    def get(self, variable):
-        '''
-        Return data from simulation file.
-        '''
-        if (not isinstance(variable, str)):
-            raise TypeError("input parameter 'variable' must be a string")
-        
-        data = self.call("EM_get", key=variable, sim=self.dsim)
-        
-        return data
-    
-    def inspect(self):
-        '''
-        Return list of keys from available data in simulation file.
-        '''
-        
-        fkeys = self.call("EM_inspect", sim=self.dsim)
-        
-        return fkeys
-    
     def close(self, **kwargs):
         '''
         Send saving options to EMode and close the connection.
@@ -169,7 +156,7 @@ class EMode:
             self.s.sendall(msg)
             while True:
                 time.sleep(0.01)
-                if self.proc.poll() is None:
+                if (self.proc.poll() is None) or (self.proc.poll() is self.proc.returncode):
                     break
             time.sleep(1.0)
             self.s.shutdown(socket.SHUT_RDWR)
@@ -179,6 +166,16 @@ class EMode:
         self.status = 'closed'
         return
     
+    def __getattr__(self, name):
+        def wrapper(*args, **kwargs):
+            try:
+                if args: kwargs['key'] = args[0]
+                result = self.call('EM_'+name, **kwargs)
+                return result
+            except:
+                raise RuntimeError("Unknown function or parameter.")
+        return wrapper
+    
     def close_atexit(self, **kwargs):
         if self.status == 'open':
             self.close()
@@ -206,10 +203,14 @@ def obj_hook(dct):
         return np.frombuffer(data, dct['dtype']).reshape(dct['shape'])
     return dct
 
-def open_file(sim):
+def open_file(sim='emode', simulation_name=None):
     '''
     Opens an EMode simulation file with either .eph or .mat extension.
     '''
+    if (simulation_name != None): sim = simulation_name
+    if (not isinstance(sim, str)):
+        raise TypeError("input parameter 'simulation_name' must be a string")
+    
     ext = '.eph'
     mat = '.mat'
     found = False
@@ -231,15 +232,16 @@ def open_file(sim):
     
     return f
 
-def get(variable, sim='emode'):
+def get(variable, sim='emode', simulation_name=None):
     '''
     Return data from simulation file.
     '''
     if (not isinstance(variable, str)):
         raise TypeError("input parameter 'variable' must be a string")
     
+    if (simulation_name != None): sim = simulation_name
     if (not isinstance(sim, str)):
-        raise TypeError("input parameter 'sim' must be a string")
+        raise TypeError("input parameter 'simulation_name' must be a string")
     
     f = open_file(sim=sim)
     
@@ -251,12 +253,13 @@ def get(variable, sim='emode'):
     
     return data
 
-def inspect(sim='emode'):
+def inspect(sim='emode', simulation_name=None):
     '''
     Return list of keys from available data in simulation file.
     '''
+    if (simulation_name != None): sim = simulation_name
     if (not isinstance(sim, str)):
-        raise TypeError("input parameter 'sim' must be a string")
+        raise TypeError("input parameter 'simulation_name' must be a string")
     
     f = open_file(sim=sim)
     
